@@ -4,22 +4,26 @@ from collections import defaultdict
 from decimal import Decimal
 from itertools import groupby
 
-from sql.aggregate import Sum, Count
+from sql.aggregate import Count, Sum
+from sql.functions import CharLength
+from sql.operators import Abs
 
 from trytond.i18n import gettext
 from trytond.model import (
-    Workflow, ModelView, ModelSQL, DeactivableMixin, fields)
+    DeactivableMixin, Index, ModelSQL, ModelView, Workflow, fields)
 from trytond.model.exceptions import AccessError
+from trytond.modules.company.model import (
+    employee_field, reset_employee, set_employee)
+from trytond.modules.currency.fields import Monetary
+from trytond.pool import Pool
 from trytond.pyson import Eval, If
 from trytond.rpc import RPC
-from trytond.tools import reduce_ids, grouped_slice, cursor_dict
+from trytond.tools import (
+    cursor_dict, grouped_slice, reduce_ids, sortable_values)
 from trytond.transaction import Transaction
-from trytond.tools import sortable_values
-from trytond.wizard import Wizard, StateView, StateAction, Button
-from trytond.pool import Pool
-from trytond.modules.company.model import (
-    employee_field, set_employee, reset_employee)
-from .exceptions import OverpayWarning
+from trytond.wizard import StateAction, Wizard
+
+from .exceptions import OverpayWarning, ReconciledWarning
 
 KINDS = [
     ('payable', 'Payable'),
@@ -32,8 +36,7 @@ class Journal(DeactivableMixin, ModelSQL, ModelView):
     __name__ = 'account.payment.journal'
     name = fields.Char('Name', required=True)
     currency = fields.Many2One('currency.currency', 'Currency', required=True)
-    company = fields.Many2One('company.company', 'Company', required=True,
-        select=True)
+    company = fields.Many2One('company.company', "Company", required=True)
     process_method = fields.Selection([
             ('manual', 'Manual'),
             ], 'Process Method', required=True)
@@ -62,35 +65,31 @@ class Group(ModelSQL, ModelView):
     number = fields.Char('Number', required=True, readonly=True)
     company = fields.Many2One(
         'company.company', "Company",
-        required=True, readonly=True, select=True)
+        required=True, readonly=True)
     journal = fields.Many2One('account.payment.journal', 'Journal',
         required=True, readonly=True, domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
+            ])
     kind = fields.Selection(KINDS, 'Kind', required=True, readonly=True)
     payments = fields.One2Many(
         'account.payment', 'group', 'Payments', readonly=True,
         domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
-    currency_digits = fields.Function(fields.Integer("Currency Digits"),
-        'on_change_with_currency_digits')
+            ('journal', '=', Eval('journal', -1)),
+            ])
+    currency = fields.Function(fields.Many2One(
+            'currency.currency', "Currency"),
+        'on_change_with_currency', searcher='search_currency')
     payment_count = fields.Function(fields.Integer(
             "Payment Count",
             help="The number of payments in the group."),
         'get_payment_aggregated')
-    payment_amount = fields.Function(fields.Numeric(
-            "Payment Total Amount",
-            digits=(16, Eval('currency_digits', 2)),
-            depends=['currency_digits'],
+    payment_amount = fields.Function(Monetary(
+            "Payment Total Amount", currency='currency', digits='currency',
             help="The sum of all payment amounts."),
         'get_payment_aggregated')
-    payment_amount_succeeded = fields.Function(fields.Numeric(
-            "Payment Succeeded",
-            digits=(16, Eval('currency_digits', 2)),
-            depends=['currency_digits'],
+    payment_amount_succeeded = fields.Function(Monetary(
+            "Payment Succeeded", currency='currency', digits='currency',
             help="The sum of the amounts of the successful payments."),
         'get_payment_aggregated')
     payment_complete = fields.Function(fields.Boolean(
@@ -98,19 +97,45 @@ class Group(ModelSQL, ModelView):
             help="All the payments in the group are complete."),
         'get_payment_aggregated', searcher='search_complete')
 
-    @classmethod
-    def __register__(cls, module_name):
-        table_h = cls.__table_handler__(module_name)
+    process_method = fields.Function(
+        fields.Selection('get_process_methods', "Process Method"),
+        'on_change_with_process_method', searcher='search_process_method')
 
-        # Migration from 3.8: rename reference into number
-        if table_h.column_exist('reference'):
-            table_h.column_rename('reference', 'number')
-        super(Group, cls).__register__(module_name)
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls._buttons.update(
+            succeed={
+                'invisible': Eval('payment_complete', False),
+                'depends': ['payment_complete', 'process_method'],
+                },
+            )
+
+    @classmethod
+    def order_number(cls, tables):
+        table, _ = tables[None]
+        return [CharLength(table.number), table.number]
 
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
 
+    @classmethod
+    def get_process_methods(cls):
+        pool = Pool()
+        Journal = pool.get('account.payment.journal')
+        field_name = 'process_method'
+        return Journal.fields_get([field_name])[field_name]['selection']
+
+    @fields.depends('journal')
+    def on_change_with_process_method(self, name=None):
+        if self.journal:
+            return self.journal.process_method
+
+    @classmethod
+    def search_process_method(cls, name, clause):
+        return [('journal.' + clause[0],) + tuple(clause[1:])]
+
     @classmethod
     def create(cls, vlist):
         pool = Pool()
@@ -136,6 +161,14 @@ class Group(ModelSQL, ModelView):
         default.setdefault('payments', None)
         return super(Group, cls).copy(groups, default=default)
 
+    @classmethod
+    @ModelView.button
+    def succeed(cls, groups):
+        pool = Pool()
+        Payment = pool.get('account.payment')
+        payments = sum((g.payments for g in groups), ())
+        Payment.succeed(payments)
+
     @classmethod
     def _get_complete_states(cls):
         return ['succeeded', 'failed']
@@ -226,45 +259,48 @@ class Group(ModelSQL, ModelView):
             return []
 
     @fields.depends('journal')
-    def on_change_with_currency_digits(self, name=None):
-        if self.journal:
-            return self.journal.currency.digits
+    def on_change_with_currency(self, name=None):
+        return self.journal.currency if self.journal else None
+
+    @classmethod
+    def search_currency(cls, name, clause):
+        return [('journal.' + clause[0],) + tuple(clause[1:])]
 
 
 _STATES = {
     'readonly': Eval('state') != 'draft',
     }
-_DEPENDS = ['state']
 
 
 class Payment(Workflow, ModelSQL, ModelView):
     'Payment'
     __name__ = 'account.payment'
     company = fields.Many2One(
-        'company.company', "Company", required=True, select=True,
-        states=_STATES, depends=_DEPENDS)
+        'company.company', "Company", required=True, states=_STATES)
     journal = fields.Many2One('account.payment.journal', 'Journal',
         required=True, states=_STATES, domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=_DEPENDS + ['company'])
+            ])
     currency = fields.Function(fields.Many2One('currency.currency',
             'Currency'), 'on_change_with_currency',
         searcher='search_currency')
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'on_change_with_currency_digits')
     kind = fields.Selection(KINDS, 'Kind', required=True,
-        states=_STATES, depends=_DEPENDS)
+        states=_STATES)
     party = fields.Many2One(
         'party.party', "Party", required=True, states=_STATES,
         context={
             'company': Eval('company', -1),
             },
-        depends=_DEPENDS + ['company'])
-    date = fields.Date('Date', required=True, states=_STATES, depends=_DEPENDS)
-    amount = fields.Numeric('Amount', required=True,
-        digits=(16, Eval('currency_digits', 2)), states=_STATES,
-        depends=_DEPENDS + ['currency_digits'])
+        depends={'company'})
+    date = fields.Date('Date', required=True, states=_STATES)
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency', required=True,
+        states={
+            'readonly': ~Eval('state').in_(
+                If(Eval('process_method') == 'manual',
+                    ['draft', 'processing'],
+                    ['draft'])),
+            })
     line = fields.Many2One('account.move.line', 'Line', ondelete='RESTRICT',
         domain=[
             ('move.company', '=', Eval('company', -1)),
@@ -280,6 +316,7 @@ class Payment(Workflow, ModelSQL, ModelView):
             If(Eval('state') == 'draft',
                 [
                     ('reconciliation', '=', None),
+                    ('maturity_date', '!=', None),
                     ],
                 []),
             ['OR',
@@ -291,15 +328,13 @@ class Payment(Workflow, ModelSQL, ModelView):
                 ],
             ('move_state', '=', 'posted'),
             ],
-        states=_STATES, depends=_DEPENDS + ['party', 'currency', 'kind',
-            'company'])
-    description = fields.Char('Description', states=_STATES, depends=_DEPENDS)
+        states=_STATES)
+    description = fields.Char('Description', states=_STATES)
     origin = fields.Reference(
-        "Origin", selection='get_origin', select=True,
+        "Origin", selection='get_origin',
         states={
             'readonly': Eval('state') != 'draft',
-            },
-        depends=['state'])
+            })
     group = fields.Many2One('account.payment.group', 'Group', readonly=True,
         ondelete='RESTRICT',
         states={
@@ -307,11 +342,13 @@ class Payment(Workflow, ModelSQL, ModelView):
             },
         domain=[
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['state', 'company'])
+            ])
     process_method = fields.Function(
         fields.Selection('get_process_methods', "Process Method"),
         'on_change_with_process_method', searcher='search_process_method')
+    submitted_by = employee_field(
+        "Submitted by",
+        states=['submitted', 'processing', 'succeeded', 'failed'])
     approved_by = employee_field(
         "Approved by",
         states=['approved', 'processing', 'succeeded', 'failed'])
@@ -322,21 +359,46 @@ class Payment(Workflow, ModelSQL, ModelView):
         states=['failed', 'processing'])
     state = fields.Selection([
             ('draft', 'Draft'),
+            ('submitted', "Submitted"),
             ('approved', 'Approved'),
             ('processing', 'Processing'),
             ('succeeded', 'Succeeded'),
             ('failed', 'Failed'),
-            ], 'State', readonly=True, select=True)
+            ], "State", readonly=True, sort=False,
+        domain=[
+            If(Eval('kind') == 'receivable',
+                ('state', '!=', 'approved'),
+                ()),
+            ])
+
+    @property
+    def amount_line_paid(self):
+        if self.state != 'failed':
+            if self.line.second_currency:
+                payment_amount = abs(self.line.amount_second_currency)
+            else:
+                payment_amount = abs(self.line.credit - self.line.debit)
+            return max(min(self.amount, payment_amount), 0)
+        return Decimal(0)
 
     @classmethod
     def __setup__(cls):
         super(Payment, cls).__setup__()
+        t = cls.__table__()
+        cls._sql_indexes.add(
+            Index(
+                t, (t.state, Index.Equality()),
+                where=t.state.in_([
+                        'draft', 'submitted', 'approved', 'processing'])))
         cls._order.insert(0, ('date', 'DESC'))
         cls._transitions |= set((
-                ('draft', 'approved'),
+                ('draft', 'submitted'),
+                ('submitted', 'approved'),
+                ('submitted', 'processing'),
                 ('approved', 'processing'),
                 ('processing', 'succeeded'),
                 ('processing', 'failed'),
+                ('submitted', 'draft'),
                 ('approved', 'draft'),
                 ('succeeded', 'failed'),
                 ('succeeded', 'processing'),
@@ -345,15 +407,37 @@ class Payment(Workflow, ModelSQL, ModelView):
                 ))
         cls._buttons.update({
                 'draft': {
-                    'invisible': Eval('state') != 'approved',
+                    'invisible': ~Eval('state').in_(['submitted', 'approved']),
                     'icon': 'tryton-back',
                     'depends': ['state'],
                     },
-                'approve': {
+                'submit': {
                     'invisible': Eval('state') != 'draft',
                     'icon': 'tryton-forward',
                     'depends': ['state'],
                     },
+                'approve': {
+                    'invisible': (
+                        (Eval('state') != 'submitted')
+                        | (Eval('kind') == 'receivable')),
+                    'icon': 'tryton-forward',
+                    'depends': ['state', 'kind'],
+                    },
+                'process_wizard': {
+                    'invisible': ~(
+                        (Eval('state') == 'approved')
+                        | ((Eval('state') == 'submitted')
+                            & (Eval('kind') == 'receivable'))),
+                    'icon': 'tryton-launch',
+                    'depends': ['state', 'kind'],
+                    },
+                'proceed': {
+                    'invisible': (
+                        ~Eval('state').in_(['succeeded', 'failed'])
+                        | (Eval('process_method') != 'manual')),
+                    'icon': 'tryton-back',
+                    'depends': ['state', 'process_method'],
+                    },
                 'succeed': {
                     'invisible': ~Eval('state').in_(
                         ['processing', 'failed']),
@@ -392,19 +476,22 @@ class Payment(Workflow, ModelSQL, ModelView):
 
     @fields.depends('journal')
     def on_change_with_currency(self, name=None):
-        if self.journal:
-            return self.journal.currency.id
-
-    @fields.depends('journal')
-    def on_change_with_currency_digits(self, name=None):
-        if self.journal:
-            return self.journal.currency.digits
-        return 2
+        return self.journal.currency if self.journal else None
 
     @classmethod
     def search_currency(cls, name, clause):
         return [('journal.' + clause[0],) + tuple(clause[1:])]
 
+    @classmethod
+    def order_amount(cls, tables):
+        table, _ = tables[None]
+        context = Transaction().context
+        column = cls.amount.sql_column(table)
+        if isinstance(context.get('amount_order'), Decimal):
+            return [Abs(column - abs(context['amount_order']))]
+        else:
+            return [column]
+
     @fields.depends('kind')
     def on_change_kind(self):
         self.line = None
@@ -448,6 +535,15 @@ class Payment(Workflow, ModelSQL, ModelView):
         field_name = 'process_method'
         return Journal.fields_get([field_name])[field_name]['selection']
 
+    @classmethod
+    def view_attributes(cls):
+        context = Transaction().context
+        attributes = super().view_attributes()
+        if context.get('kind') == 'receivable':
+            attributes.append(
+                ('/tree//button[@name="approve"]', 'tree_invisible', True))
+        return attributes
+
     @classmethod
     def copy(cls, payments, default=None):
         if default is None:
@@ -472,15 +568,27 @@ class Payment(Workflow, ModelSQL, ModelView):
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
-    @reset_employee('approved_by', 'succeeded_by', 'failed_by')
+    @reset_employee('submitted_by', 'approved_by', 'succeeded_by', 'failed_by')
     def draft(cls, payments):
         pass
 
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('submitted')
+    @set_employee('submitted_by')
+    def submit(cls, payments):
+        cls._check_reconciled(payments)
+
     @classmethod
     @ModelView.button
     @Workflow.transition('approved')
     @set_employee('approved_by')
     def approve(cls, payments):
+        cls._check_reconciled(payments)
+
+    @classmethod
+    @ModelView.button_action('account_payment.act_process_payments')
+    def process_wizard(cls, payments):
         pass
 
     @classmethod
@@ -504,9 +612,12 @@ class Payment(Workflow, ModelSQL, ModelView):
             return group
 
     @classmethod
+    @ModelView.button
     @Workflow.transition('processing')
     def proceed(cls, payments):
         assert all(p.group for p in payments)
+        cls._check_reconciled(
+            [p for p in payments if p.state not in {'succeeded', 'failed'}])
 
     @classmethod
     @ModelView.button
@@ -522,20 +633,25 @@ class Payment(Workflow, ModelSQL, ModelView):
     def fail(cls, payments):
         pass
 
-
-class ProcessPaymentStart(ModelView):
-    'Process Payment'
-    __name__ = 'account.payment.process.start'
+    @classmethod
+    def _check_reconciled(cls, payments):
+        pool = Pool()
+        Warning = pool.get('res.user.warning')
+        for payment in payments:
+            if payment.line and payment.line.reconciliation:
+                key = Warning.format('submit_reconciled', [payment])
+                if Warning.check(key):
+                    raise ReconciledWarning(
+                        key, gettext(
+                            'account_payment.msg_payment_reconciled',
+                            payment=payment.rec_name,
+                            line=payment.line.rec_name))
 
 
 class ProcessPayment(Wizard):
     'Process Payment'
     __name__ = 'account.payment.process'
-    start = StateView('account.payment.process.start',
-        'account_payment.payment_process_start_view_form', [
-            Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Process', 'process', 'tryton-ok', default=True),
-            ])
+    start_state = 'process'
     process = StateAction('account_payment.act_payment_group_form')
 
     def _group_payment_key(self, payment):
@@ -556,6 +672,11 @@ class ProcessPayment(Wizard):
         Warning = pool.get('res.user.warning')
         payments = self.records
 
+        payments = [
+            p for p in payments
+            if p.state == 'approved'
+            or (p.state == 'submitted' and p.kind == 'receivable')]
+
         for payment in payments:
             if payment.line and payment.line.payment_amount < 0:
                 if Warning.check(str(payment)):
